package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.additional.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 聊天群聊消息操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "聊天群聊消息管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourImMessage")
public class TourImMessageController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private TourImMessageService tourImMessageService;

    /**
     * 新增聊天群聊消息数据。
     *
     * @param tourImMessageDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourImMessageDto.id", "tourImMessageDto.searchString"})
    @SaCheckPermission("tourImMessage.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourImMessageDto tourImMessageDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourImMessageDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourImMessage tourImMessage = MyModelUtil.copyTo(tourImMessageDto, TourImMessage.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourImMessageService.verifyRelatedData(tourImMessage, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourImMessage = tourImMessageService.saveNew(tourImMessage);
        return ResponseResult.success(tourImMessage.getId());
    }

    /**
     * 更新聊天群聊消息数据。
     *
     * @param tourImMessageDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"tourImMessageDto.searchString"})
    @SaCheckPermission("tourImMessage.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourImMessageDto tourImMessageDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourImMessageDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourImMessage tourImMessage = MyModelUtil.copyTo(tourImMessageDto, TourImMessage.class);
        TourImMessage originalTourImMessage = tourImMessageService.getById(tourImMessage.getId());
        if (originalTourImMessage == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourImMessageService.verifyRelatedData(tourImMessage, originalTourImMessage);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourImMessageService.update(tourImMessage, originalTourImMessage)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除聊天群聊消息数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourImMessage.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除聊天群聊消息数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourImMessage.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的聊天群聊消息列表。
     *
     * @param tourImMessageDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourImMessage.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourImMessageVo>> list(
            @MyRequestBody TourImMessageDto tourImMessageDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourImMessage tourImMessageFilter = MyModelUtil.copyTo(tourImMessageDtoFilter, TourImMessage.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourImMessage.class);
        List<TourImMessage> tourImMessageList =
                tourImMessageService.getTourImMessageListWithRelation(tourImMessageFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourImMessageList, TourImMessageVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourImMessage.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateTime");
        ignoreFieldSet.add("updateUserId");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(TourImMessage.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<TourImMessage> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, TourImMessage.class, translatedDictFieldSet);
        CallResult result = tourImMessageService.verifyImportList(dataList, translatedDictFieldSet);
        if (!result.isSuccess()) {
            // result中返回了具体的验证失败对象，如果需要返回更加详细的错误，可根据实际情况手动修改。
            return ResponseResult.errorFrom(result);
        }
        tourImMessageService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的聊天群聊消息列表。
     *
     * @param tourImMessageDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("tourImMessage.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody TourImMessageDto tourImMessageDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        TourImMessage tourImMessageFilter = MyModelUtil.copyTo(tourImMessageDtoFilter, TourImMessage.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourImMessage.class);
        List<TourImMessage> resultList =
                tourImMessageService.getTourImMessageListWithRelation(tourImMessageFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(14);
        headerMap.put("id", "id");
        headerMap.put("messageContent", "消息内容");
        headerMap.put("messageTypeDictMap.name", "消息类型 0文字 1文件 2系统消息");
        headerMap.put("groupIdDictMap.name", "聊天群组id");
        headerMap.put("noticeTypeDictMap.name", "通知消息类型 1.单聊消息 2 群聊消息 3系统消息 4关注信息 5点赞 6评论 7评论区艾特 8文章内艾特 9访问");
        headerMap.put("isTopDictMap.name", "消息置顶/公告 0不置顶 1置顶");
        headerMap.put("revocationTagDictMap.name", "撤回标记 0不撤回 1撤回");
        headerMap.put("deletedByUsers", "已经删除该条消息的用户");
        headerMap.put("readByUsers", "已读该条消息的用户");
        headerMap.put("dataState", "删除标记 1正常 -1删除");
        headerMap.put("createUserId", "创建人id");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateTime", "更新时间");
        headerMap.put("updateUserId", "更新人id");
        ExportUtil.doExport(resultList, headerMap, "tourImMessage.xlsx");
    }

    /**
     * 查看指定聊天群聊消息对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourImMessage.view")
    @GetMapping("/view")
    public ResponseResult<TourImMessageVo> view(@RequestParam Long id) {
        TourImMessage tourImMessage = tourImMessageService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourImMessage == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourImMessageVo tourImMessageVo = MyModelUtil.copyTo(tourImMessage, TourImMessageVo.class);
        return ResponseResult.success(tourImMessageVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourImMessage originalTourImMessage = tourImMessageService.getById(id);
        if (originalTourImMessage == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourImMessageService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
