package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 游记评论中提到的用户操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "游记评论中提到的用户管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourTravelNotesCommentMentionUser")
public class TourTravelNotesCommentMentionUserController {

    @Autowired
    private TourTravelNotesCommentMentionUserService tourTravelNotesCommentMentionUserService;

    /**
     * 新增游记评论中提到的用户数据。
     *
     * @param tourTravelNotesCommentMentionUserDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourTravelNotesCommentMentionUserDto.id"})
    @SaCheckPermission("tourTravelNotesCommentMentionUser.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourTravelNotesCommentMentionUserDto tourTravelNotesCommentMentionUserDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesCommentMentionUserDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesCommentMentionUser tourTravelNotesCommentMentionUser = MyModelUtil.copyTo(tourTravelNotesCommentMentionUserDto, TourTravelNotesCommentMentionUser.class);
        tourTravelNotesCommentMentionUser = tourTravelNotesCommentMentionUserService.saveNew(tourTravelNotesCommentMentionUser);
        return ResponseResult.success(tourTravelNotesCommentMentionUser.getId());
    }

    /**
     * 更新游记评论中提到的用户数据。
     *
     * @param tourTravelNotesCommentMentionUserDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesCommentMentionUser.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourTravelNotesCommentMentionUserDto tourTravelNotesCommentMentionUserDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesCommentMentionUserDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesCommentMentionUser tourTravelNotesCommentMentionUser = MyModelUtil.copyTo(tourTravelNotesCommentMentionUserDto, TourTravelNotesCommentMentionUser.class);
        TourTravelNotesCommentMentionUser originalTourTravelNotesCommentMentionUser = tourTravelNotesCommentMentionUserService.getById(tourTravelNotesCommentMentionUser.getId());
        if (originalTourTravelNotesCommentMentionUser == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesCommentMentionUserService.update(tourTravelNotesCommentMentionUser, originalTourTravelNotesCommentMentionUser)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除游记评论中提到的用户数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesCommentMentionUser.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除游记评论中提到的用户数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesCommentMentionUser.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的游记评论中提到的用户列表。
     *
     * @param tourTravelNotesCommentMentionUserDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourTravelNotesCommentMentionUser.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourTravelNotesCommentMentionUserVo>> list(
            @MyRequestBody TourTravelNotesCommentMentionUserDto tourTravelNotesCommentMentionUserDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourTravelNotesCommentMentionUser tourTravelNotesCommentMentionUserFilter = MyModelUtil.copyTo(tourTravelNotesCommentMentionUserDtoFilter, TourTravelNotesCommentMentionUser.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourTravelNotesCommentMentionUser.class);
        List<TourTravelNotesCommentMentionUser> tourTravelNotesCommentMentionUserList =
                tourTravelNotesCommentMentionUserService.getTourTravelNotesCommentMentionUserListWithRelation(tourTravelNotesCommentMentionUserFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourTravelNotesCommentMentionUserList, TourTravelNotesCommentMentionUserVo.class));
    }

    /**
     * 查看指定游记评论中提到的用户对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourTravelNotesCommentMentionUser.view")
    @GetMapping("/view")
    public ResponseResult<TourTravelNotesCommentMentionUserVo> view(@RequestParam Long id) {
        TourTravelNotesCommentMentionUser tourTravelNotesCommentMentionUser = tourTravelNotesCommentMentionUserService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourTravelNotesCommentMentionUser == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourTravelNotesCommentMentionUserVo tourTravelNotesCommentMentionUserVo = MyModelUtil.copyTo(tourTravelNotesCommentMentionUser, TourTravelNotesCommentMentionUserVo.class);
        return ResponseResult.success(tourTravelNotesCommentMentionUserVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourTravelNotesCommentMentionUser originalTourTravelNotesCommentMentionUser = tourTravelNotesCommentMentionUserService.getById(id);
        if (originalTourTravelNotesCommentMentionUser == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesCommentMentionUserService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
