package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 游记话题操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "游记话题管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourTravelNotesTopic")
public class TourTravelNotesTopicController {

    @Autowired
    private TourTravelNotesTopicService tourTravelNotesTopicService;

    /**
     * 新增游记话题数据。
     *
     * @param tourTravelNotesTopicDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourTravelNotesTopicDto.id"})
    @SaCheckPermission("tourTravelNotesTopic.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourTravelNotesTopicDto tourTravelNotesTopicDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesTopicDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesTopic tourTravelNotesTopic = MyModelUtil.copyTo(tourTravelNotesTopicDto, TourTravelNotesTopic.class);
        tourTravelNotesTopic = tourTravelNotesTopicService.saveNew(tourTravelNotesTopic);
        return ResponseResult.success(tourTravelNotesTopic.getId());
    }

    /**
     * 更新游记话题数据。
     *
     * @param tourTravelNotesTopicDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesTopic.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourTravelNotesTopicDto tourTravelNotesTopicDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesTopicDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesTopic tourTravelNotesTopic = MyModelUtil.copyTo(tourTravelNotesTopicDto, TourTravelNotesTopic.class);
        TourTravelNotesTopic originalTourTravelNotesTopic = tourTravelNotesTopicService.getById(tourTravelNotesTopic.getId());
        if (originalTourTravelNotesTopic == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesTopicService.update(tourTravelNotesTopic, originalTourTravelNotesTopic)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除游记话题数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesTopic.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除游记话题数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesTopic.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的游记话题列表。
     *
     * @param tourTravelNotesTopicDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourTravelNotesTopic.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourTravelNotesTopicVo>> list(
            @MyRequestBody TourTravelNotesTopicDto tourTravelNotesTopicDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourTravelNotesTopic tourTravelNotesTopicFilter = MyModelUtil.copyTo(tourTravelNotesTopicDtoFilter, TourTravelNotesTopic.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourTravelNotesTopic.class);
        List<TourTravelNotesTopic> tourTravelNotesTopicList =
                tourTravelNotesTopicService.getTourTravelNotesTopicListWithRelation(tourTravelNotesTopicFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourTravelNotesTopicList, TourTravelNotesTopicVo.class));
    }

    /**
     * 查看指定游记话题对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourTravelNotesTopic.view")
    @GetMapping("/view")
    public ResponseResult<TourTravelNotesTopicVo> view(@RequestParam Long id) {
        TourTravelNotesTopic tourTravelNotesTopic = tourTravelNotesTopicService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourTravelNotesTopic == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourTravelNotesTopicVo tourTravelNotesTopicVo = MyModelUtil.copyTo(tourTravelNotesTopic, TourTravelNotesTopicVo.class);
        return ResponseResult.success(tourTravelNotesTopicVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourTravelNotesTopic originalTourTravelNotesTopic = tourTravelNotesTopicService.getById(id);
        if (originalTourTravelNotesTopic == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesTopicService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
