package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 用户与评论的收藏关系表操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "用户与评论的收藏关系表管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourUserCommentCollectRelation")
public class TourUserCommentCollectRelationController {

    @Autowired
    private TourUserCommentCollectRelationService tourUserCommentCollectRelationService;

    /**
     * 新增用户与评论的收藏关系表数据。
     *
     * @param tourUserCommentCollectRelationDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourUserCommentCollectRelationDto.userId"})
    @SaCheckPermission("tourUserCommentCollectRelation.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourUserCommentCollectRelationDto tourUserCommentCollectRelationDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourUserCommentCollectRelationDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourUserCommentCollectRelation tourUserCommentCollectRelation = MyModelUtil.copyTo(tourUserCommentCollectRelationDto, TourUserCommentCollectRelation.class);
        tourUserCommentCollectRelation = tourUserCommentCollectRelationService.saveNew(tourUserCommentCollectRelation);
        return ResponseResult.success(tourUserCommentCollectRelation.getUserId());
    }

    /**
     * 更新用户与评论的收藏关系表数据。
     *
     * @param tourUserCommentCollectRelationDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourUserCommentCollectRelation.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourUserCommentCollectRelationDto tourUserCommentCollectRelationDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourUserCommentCollectRelationDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourUserCommentCollectRelation tourUserCommentCollectRelation = MyModelUtil.copyTo(tourUserCommentCollectRelationDto, TourUserCommentCollectRelation.class);
        TourUserCommentCollectRelation originalTourUserCommentCollectRelation = tourUserCommentCollectRelationService.getById(tourUserCommentCollectRelation.getUserId());
        if (originalTourUserCommentCollectRelation == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourUserCommentCollectRelationService.update(tourUserCommentCollectRelation, originalTourUserCommentCollectRelation)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除用户与评论的收藏关系表数据。
     *
     * @param userId 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourUserCommentCollectRelation.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long userId) {
        if (MyCommonUtil.existBlankArgument(userId)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(userId);
    }

    /**
     * 批量删除用户与评论的收藏关系表数据。
     *
     * @param userIdList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourUserCommentCollectRelation.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> userIdList) {
        if (MyCommonUtil.existBlankArgument(userIdList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long userId : userIdList) {
            ResponseResult<Void> responseResult = this.doDelete(userId);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的用户与评论的收藏关系表列表。
     *
     * @param tourUserCommentCollectRelationDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourUserCommentCollectRelation.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourUserCommentCollectRelationVo>> list(
            @MyRequestBody TourUserCommentCollectRelationDto tourUserCommentCollectRelationDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourUserCommentCollectRelation tourUserCommentCollectRelationFilter = MyModelUtil.copyTo(tourUserCommentCollectRelationDtoFilter, TourUserCommentCollectRelation.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourUserCommentCollectRelation.class);
        List<TourUserCommentCollectRelation> tourUserCommentCollectRelationList =
                tourUserCommentCollectRelationService.getTourUserCommentCollectRelationListWithRelation(tourUserCommentCollectRelationFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourUserCommentCollectRelationList, TourUserCommentCollectRelationVo.class));
    }

    /**
     * 查看指定用户与评论的收藏关系表对象详情。
     *
     * @param userId 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourUserCommentCollectRelation.view")
    @GetMapping("/view")
    public ResponseResult<TourUserCommentCollectRelationVo> view(@RequestParam Long userId) {
        TourUserCommentCollectRelation tourUserCommentCollectRelation = tourUserCommentCollectRelationService.getByIdWithRelation(userId, MyRelationParam.full());
        if (tourUserCommentCollectRelation == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourUserCommentCollectRelationVo tourUserCommentCollectRelationVo = MyModelUtil.copyTo(tourUserCommentCollectRelation, TourUserCommentCollectRelationVo.class);
        return ResponseResult.success(tourUserCommentCollectRelationVo);
    }

    private ResponseResult<Void> doDelete(Long userId) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourUserCommentCollectRelation originalTourUserCommentCollectRelation = tourUserCommentCollectRelationService.getById(userId);
        if (originalTourUserCommentCollectRelation == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourUserCommentCollectRelationService.remove(userId)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
